# Generated by Django 4.2.9 on 2025-01-03 16:12

from django.db import migrations, models

from funkwhale_api.federation import utils as federation_utils
from django.urls import reverse
import uuid


def insert_tracks_to_playlist(apps, playlist, uploads):
    PlaylistTrack = apps.get_model("playlists", "PlaylistTrack")
    plts = [
        PlaylistTrack(
            creation_date=playlist.creation_date,
            playlist=playlist,
            track=upload.track,
            index=0 + i,
            uuid=(new_uuid := uuid.uuid4()),
            fid=federation_utils.full_url(
                reverse(
                    "federation:music:playlist-tracks-detail",
                    kwargs={"uuid": new_uuid},
                )
            ),
        )
        for i, upload in enumerate(uploads)
        if upload.track
    ]

    return PlaylistTrack.objects.bulk_create(plts)


def migrate_libraries_to_playlist(apps, schema_editor):
    Playlist = apps.get_model("playlists", "Playlist")
    Library = apps.get_model("music", "Library")
    Actor = apps.get_model("federation", "Actor")

    for library in Library.objects.all():
        try:
            playlist = Playlist.objects.create(
                name=library.name,
                library=library,
                actor=library.actor,
                creation_date=library.creation_date,
                privacy_level=library.privacy_level,
                description=library.description,
                uuid=(new_uuid := uuid.uuid4()),
                fid=federation_utils.full_url(
                    reverse(
                        "federation:music:playlists-detail",
                        kwargs={"uuid": new_uuid},
                    )
                ),
            )
            playlist.save()

            if library.uploads.all().exists():
                uploads = library.uploads.all()
                insert_tracks_to_playlist(apps, playlist, uploads)
                playlist.library.playlist_uploads.set(uploads)
        except Exception as e:
            print(f"An error occurred during library.playlist creation {e}")
            continue

    # migrate uploads to new built-in libraries
    for actor in Actor.objects.all():
        privacy_levels = ["me", "instance", "everyone"]
        for privacy_level in privacy_levels:
            build_in_lib = Library.objects.create(
                actor=actor,
                privacy_level=privacy_level,
                name=privacy_level,
                uuid=(new_uuid := uuid.uuid4()),
                fid=federation_utils.full_url(
                    reverse(
                        "federation:music:libraries-detail",
                        kwargs={"uuid": new_uuid},
                    )
                ),
            )
            for library in actor.libraries.filter(privacy_level=privacy_level):
                library.uploads.all().update(library=build_in_lib)


class Migration(migrations.Migration):
    dependencies = [
        ("music", "0060_empty_for_test"),
        ("playlists", "0009_playlist_library"),
    ]
    operations = [
        migrations.AddField(
            model_name="upload",
            name="playlist_libraries",
            field=models.ManyToManyField(
                blank=True,
                related_name="playlist_uploads",
                to="music.library",
            ),
        ),
        migrations.RunPython(
            migrate_libraries_to_playlist, reverse_code=migrations.RunPython.noop
        ),
    ]
