import django.db.models.deletion
from django.db import migrations, models


# to do : test migration
def add_uploads_to_pl_library(playlist, library):
    for plt in playlist.playlist_tracks.all():
        for upload in plt.track.uploads.filter(library__actor=playlist.actor):
            library.uploads.add(upload)


def create_playlist_libraries(apps, schema_editor):
    Playlist = apps.get_model("playlists", "Playlist")
    Library = apps.get_model("music", "Library")

    for playlist in Playlist.objects.all():
        library = playlist.library
        if library is None:
            try:
                library = Library.objects.create(
                    name=playlist.name, privacy_level="me", actor=playlist.actor
                )
                library.save()
                playlist.library = library
                playlist.save()
                add_uploads_to_pl_library(playlist, library)
            except Exception as e:
                print(f"An error occurred during playlist.library creation {e}")
                continue


class Migration(migrations.Migration):
    dependencies = [
        ("playlists", "0008_playlist_library_drop"),
    ]

    operations = [
        migrations.AddField(
            model_name="playlist",
            name="library",
            field=models.OneToOneField(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.CASCADE,
                related_name="playlist",
                to="music.library",
            ),
        ),
        migrations.RunPython(
            create_playlist_libraries, reverse_code=migrations.RunPython.noop
        ),
        migrations.AlterField(
            model_name="playlist",
            name="library",
            field=models.OneToOneField(
                on_delete=django.db.models.deletion.CASCADE,
                related_name="playlist",
                to="music.library",
            ),
        ),
    ]
